/**
 * Copyright (c) 2009 Software Architecture Group
 *                    Hasso-Plattner-Institute, Potsdam, Germany
 *  
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */
//package jcop.lang;
package contextj.lang;
import java.util.ArrayList;



public class Composition extends ArrayList<Layer>  {

	 
	private static Composition comp = new Composition();
	
	public static final ThreadLocal<Composition> current = 
		new ThreadLocal<Composition>() {
			protected Composition initialValue() {
				return comp;
			}
		};
		
		
	public static Composition current() {
		return current.get();
	}
	
	public Layer firstLayer() {
		return get(0);
	}
	
	public Composition() {
		addLayer(new Layer("root"));
	}	


	public Composition activateLayer(Layer _layer) {
		Composition old = (Composition)this.clone(); 
		addLayer(_layer);		
		return old;
	}	
	
	public Composition activateLayer(Iterable<Layer> layerList) {
		Composition old = (Composition)this.clone();		
		for (Layer _layer : layerList) 
			addLayer(_layer);	
		return old;
	}
	
	public Composition activateLayer(Layer[] layerList) {
		Composition old = (Composition)this.clone();
		for (Layer _layer : layerList) 
		  addLayer(_layer);
		return old;
	}
	
	
	//deactivates the last activated layer
	public Composition deactivateLayer(Layer _layer) {
		Composition old = (Composition)this.clone();
		remove(_layer);
		return old;
	}
	
	public static void setComposition(Composition comp) {
		current.set(comp);
	}
	
    public Layer[] getLayer() {
    	return (Layer[])this.toArray(new Layer[]{});
    }
	
	
//	
//	public static void deactivateLayer(Layer _layer) {		
//		if(_layer != null) {
//			Layer formerOuterMostLayer = activeLayers.get().getNext();
//			if(formerOuterMostLayer != null) {
//				//_layer.broadcastLayerDeactivation();
//				activeLayers.set(formerOuterMostLayer);				
//			}
//		}	
//	}	
	
	public Layer next(Layer current) {
		int pos = indexOf(current);
		return get(pos + 1);
	}
	
	private void addLayer(Layer _layer) {
		if (_layer != null) {
			if(contains(_layer)) 
				remove(_layer);
			add(0, _layer);
		}		
	}
	

	
	@Override
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append("[");
		for (Layer _layer : this) {
			buffer.append(_layer);
			buffer.append(", ");
		}
		if (buffer.length() > 0)
			buffer.delete(buffer.length() - 2, buffer.length());
		buffer.append("]");
		return buffer.toString();
	}
	

}
