/**
 * Copyright (c) 2009 Software Architecture Group
 *                    Hasso-Plattner-Institute, Potsdam, Germany
 *  
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */
//package jcop.lang;
package contextj.lang;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import jcop.Globals;



public class Layer {
	public String name;
	public static List<Layer> layers = new ArrayList<Layer>();
//	static{
//		layeredObjects = new ArrayList<Layered>();
//	}

	
	/*
	 * Reflective API
	 */
	public static Layer forName(String identifier) {		
		if (identifier == null || identifier.equals("null")) 
			return null;		
		String className = "jcop.lang." + identifier;		
		return initLayerClass(className);
	}
	
	/*
	 * Reflective API
	 */
	public static Layer[] getLayers() {	
		
		return layers.toArray(new Layer[]{});
	}
	
	class MapFunction {
		  void map(Layer o) {}
	    }
	
	
	/*
	 * Reflective API
	 */
	public boolean isActive() {
		return Composition.current().contains(this);
	}
	/*
	 * Reflective API
	 */
	public boolean isAccessedBefore(Layer l) {
		Composition current = Composition.current();
		if (!current.contains(this))
			return false;
		if (!current.contains(l))
			return true;
		return (current.indexOf(this) < current.indexOf(l));	
	}
	
	private static Layer initLayerClass(String className) {
		Class c;
	    try {
			c = Class.forName(className);			
			return (Layer)c.newInstance();
		} 
		catch (ClassNotFoundException e) {
			System.err.println("cannot instantiate layer '" + className + "'");
		}
		catch (IllegalAccessException e) {
			System.err.println("cannot access layer '" + className + "'");
		}
		catch (InstantiationException e) {
			System.err.println("cannot instantiate layer '" + className + "'");
		}	
		return null;
	}
	
//	public Layer newInstance(){ 
//		throw new RuntimeException("method should be overridden");
//	}
	
	public Layer(String name){
		this.name = name;
		this.partialMethodSignatures = new java.util.Hashtable<String, PartialMethod>();
		layers.add(this);
	}
	
	
	public String toString() {
		return name;
	}
	
	public java.util.Hashtable<String, PartialMethod> partialMethodSignatures;
	
	/*
	 * Reflective API
	 */
	public boolean providesPartialMethod(String signature) {
		return partialMethodSignatures.contains(signature);
	}

	/*
	 * Reflective API
	 */
	public PartialMethod getPartialMethod(String signature) {
		return partialMethodSignatures.get(signature);
	}

	/*
	 * Reflective API
	 */
	//	public PartialMethod getPartialMethod(Method m) {
	//		return partialMethodSignatures.get(<get signature of m>);
	//	}	
	
	/*
	 * Reflective API
	 */
	public Composition getComposition() {
		return Composition.current();
	}
	
	/*
	 * Reflective API
	 */
	public PartialMethod[] getPartialMethods() {
		return partialMethodSignatures.values().toArray(new PartialMethod[]{});
	}
	
//	private static ArrayList<Layered> layeredObjects;
//	
//	public static void registerLayeredObject(Layered layeredObject){
//		layeredObjects.add(layeredObject);		
//	}
//	
//	public static void broadcastLayerActivation() {		
//		for (Layered layeredObject : layeredObjects) {
//			layeredObject.onLayerActivation();
//		}
//	}
//	public static void broadcastLayerDeactivation() {
//		for (Layered layeredObject : layeredObjects) {
//			layeredObject.onLayerDeactivation();
//		}
//	}
//	public static Hashtable<Integer, Object> advice2ObjectTable =
//		new Hashtable<Integer, Object>();
//	
//	public static void addAdviceToObjectRelationFor(int id, Object o) {		
//		advice2ObjectTable.put(id,o);
//			
//		
//	}
	
}