/**
 * Copyright (c) 2009 Software Architecture Group
 *                    Hasso-Plattner-Institute, Potsdam, Germany
 *  
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */
package jcop.compiler;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import jcop.Globals.CompilerOps;

public class ArgumentParser {

	private String rootDir;
	private List<String> args;
	
	public ArgumentParser(String[] args) {
		this.rootDir = System.getProperty("user.dir");
		this.args = new ArrayList<String>(Arrays.asList(args));	
		this.args = concatenateQuotedValues(this.args);		
		this.args = splitRootDirFromClassPath(this.args);		
	}
	

	private List<String> splitRootDirFromClassPath(List<String> args) {
		String filePath = args.remove(args.size()-1);
		int lastFolderSeparator = filePath.lastIndexOf(File.separator) + 1;
		if(lastFolderSeparator > 1) {
			rootDir = filePath.substring(0, lastFolderSeparator - 1);
			String classPath = filePath.substring(lastFolderSeparator);		
			args.add(CompilerOps.rootDir);
			args.add(rootDir);
			args.add(classPath);
		}
		return args;
	}


	private List<String> concatenateQuotedValues(List<String> args) {
		for (int i = 0; i < args.size(); i++) {
			String arg = args.get(i);
			if (beginsButNotEndsWithQuotation(arg)) {
				List<String> newArgs = concatenateArgsAt(args, i);
				return concatenateQuotedValues(newArgs);
			}			
		}
		return args;
	}


	private List<String> concatenateArgsAt(List<String> args, int pos) {
		String arg = args.get(pos);
		String nextArg = args.get(pos);
	    String newArg = arg + "" + nextArg;
	    args.remove(pos);
	    args.remove(pos);
	    args.add(pos, newArg);
	    return args;
	}


	private boolean beginsButNotEndsWithQuotation(String arg) {
		return (arg.startsWith("\"") && !arg.endsWith("\""));		
	}


	public String[] getParsedArgs() {
		return args.toArray(new String[]{});
//		String classPath = parseClassPath();		
//		
//		
//		
//		if(lastFolderSeparator > 1) {
//			rootDir = classPath.substring(0, lastFolderSeparator - 1);
//		    classPath = classPath.substring(lastFolderSeparator);		
//			newArgs = new String[args.length + 2];			
//			System.arraycopy(args, 0, newArgs, 0, args.length);
//			newArgs[newArgs.length - 3] = "-rootDir";
//			newArgs[newArgs.length - 2] = rootDir;
//			newArgs[newArgs.length - 1] = classPath;
//		}
//		else {
//			newArgs = args;
//		}	
//		return newArgs;				
	}
	
	private String parseClassPath() {	
		return args.get(args.size()-1);
//		String pathPart = args.get(args.size() - 1);
//		StringBuffer parts = new StringBuffer();
//		parts.append(pathPart);
//		
//		if (isLastPart(pathPart)) {			
//			for (int i = args.length - 2; i >= 0; i++) {
//				parts.append(args[i]);
//				if (args[i].startsWith("\""))
//					break;
//			}			
//		}		
//		return parts.toString();
	}	
	
//	private boolean isLastPart(String pathPart) {
//		return (pathPart.endsWith("\"") && !pathPart.startsWith("\""));		
//	}
}
